import java.util.*;
import java.awt.*;
import java.awt.event.*;

class Calculation {
	public static final int GENERATION = 30;
	public static final double LENGTH_OF_CYCLE = 1.0;
	public static final double LENGTH_OF_PNRD = 0.150;
	public static final int STEPS = 5000;

	public static final double TRANSCRIPTION_RATE = 1800.0;
	public static final double TRANSCRIPTION_DELAY = 0.0050;
	public static final double TRANSLATION_RATE = 120.0;
	public static final double TRANSLATION_DELAY = 0.0050;
	public static final double HIGH_COPY = 0.10;
	public static final double LOW_COPY = 0.010;

	public static final double K_sigmaA_pA = 1.0;
	public static final double K_sigmaB_pB = 1.0;
	public static final double K_sigmaC_pC = 1.0;
	public static final double K_sigmaA_antiA = 1.0;
	public static final double K_sigmaB_antiB = 1.0;
	public static final double K_sigmaC_antiC = 1.0;
	public static final double K_araC_pbad = 0.010;
	public static final double K_araC_ARABINOSE = 0.010;
	public static final double K_triggerRNA_toehold = 1.0;

	public static final double ARAC = 1.0;

	public static final double HL_sigmaA = 0.15;
	public static final double HL_sigmaB = 0.15;
	public static final double HL_sigmaC = 0.15;
	public static final double HL_antiA = 0.15;
	public static final double HL_antiB = 0.15;
	public static final double HL_antiC = 0.15;
	public static final double HL_ARABINOSE = 0.050;
	public static final double HL_GFP = 0.250;
	public static final double HL_RFP = 0.250;
	public static final double HL_CFP = 0.250;
	public static final double HL_RNA = 0.050;

	public static final double MT_GFP = 0.20;
	public static final double MT_RFP = 0.20;
	public static final double MT_CFP = 0.20;
	
	public double[] sigmaA;
	public double[] sigmaB;
	public double[] sigmaC;
	public double[] antiA;
	public double[] antiB;
	public double[] antiC;
	public double[] triggerRNA;
	public double[] arabinose;
	public double[] mRNA_sigmaA;
	public double[] mRNA_sigmaB;
	public double[] mRNA_sigmaC;
	public double[] mRNA_antiA;
	public double[] mRNA_antiB;
	public double[] mRNA_antiC;
	public double[] cr_mRNA_sigmaA;
	public double[] cr_mRNA_sigmaB;
	public double[] cr_mRNA_sigmaC;
	public double[] matureGFP;
	public double[] immatureGFP;
	public double[] matureRFP;
	public double[] immatureRFP;
	public double[] matureCFP;
	public double[] immatureCFP;
	public double[] mRNA_GFP;
	public double[] mRNA_RFP;
	public double[] mRNA_CFP;

	protected Calculation() {
		double delta = 1.0/(double)STEPS;
		int length_of_array = (int)(GENERATION*LENGTH_OF_CYCLE*STEPS);
		Random rnd = new Random();
		rnd.setSeed(System.currentTimeMillis());

		this.sigmaA = new double[length_of_array];
		this.sigmaB = new double[length_of_array];
		this.sigmaC = new double[length_of_array];
		this.antiA = new double[length_of_array];
		this.antiB = new double[length_of_array];
		this.antiC = new double[length_of_array];
		this.triggerRNA = new double[length_of_array];
		this.arabinose = new double[length_of_array];
		this.mRNA_sigmaA = new double[length_of_array];
		this.mRNA_sigmaB = new double[length_of_array];
		this.mRNA_sigmaC = new double[length_of_array];
		this.mRNA_antiA = new double[length_of_array];
		this.mRNA_antiB = new double[length_of_array];
		this.mRNA_antiC = new double[length_of_array];
		this.cr_mRNA_sigmaA = new double[length_of_array];
		this.cr_mRNA_sigmaB = new double[length_of_array];
		this.cr_mRNA_sigmaC = new double[length_of_array];
		this.matureGFP = new double[length_of_array];
		this.immatureGFP = new double[length_of_array];
		this.matureRFP = new double[length_of_array];
		this.immatureRFP = new double[length_of_array];
		this.matureCFP = new double[length_of_array];
		this.immatureCFP = new double[length_of_array];
		this.mRNA_GFP = new double[length_of_array];
		this.mRNA_RFP = new double[length_of_array];
		this.mRNA_CFP = new double[length_of_array];

		this.arabinose[0] = 1.0;

		for(int i = 0; i < length_of_array - 1; i++) {
			int delay_tc = (int)((TRANSCRIPTION_DELAY*STEPS)*(1.0 + 0.10*rnd.nextGaussian()));
			delay_tc = (i - delay_tc < 0) ? i : delay_tc;
			int delay_tl = (int)((TRANSLATION_DELAY*STEPS)*(1.0 + 0.10*rnd.nextGaussian()));
			delay_tl = (i - delay_tl < 0) ? i : delay_tl;
			double rTRANSCRIPTION_RATE = TRANSCRIPTION_RATE*(1.0 + 0.10*rnd.nextGaussian());
			double rTRANSLATION_RATE = TRANSLATION_RATE*(1.0 + 0.10*rnd.nextGaussian());

			this.sigmaA[i + 1] = this.sigmaA[i] + delta*(rTRANSLATION_RATE*(this.mRNA_sigmaA[i - delay_tl] +
				this.cr_mRNA_sigmaA[i - delay_tl]*(this.triggerRNA[i - delay_tl]/(K_triggerRNA_toehold + this.triggerRNA[i - delay_tl]))) -
				(Math.log(2.0)/HL_sigmaA)*this.sigmaA[i]);
			this.sigmaB[i + 1] = this.sigmaB[i] + delta*(rTRANSLATION_RATE*(this.mRNA_sigmaB[i - delay_tl] +
				this.cr_mRNA_sigmaB[i - delay_tl]*(this.triggerRNA[i - delay_tl]/(K_triggerRNA_toehold + this.triggerRNA[i - delay_tl]))) -
				(Math.log(2.0)/HL_sigmaB)*this.sigmaB[i]);
			this.sigmaC[i + 1] = this.sigmaC[i] + delta*(rTRANSLATION_RATE*(this.mRNA_sigmaC[i - delay_tl] +
				this.cr_mRNA_sigmaC[i - delay_tl]*(this.triggerRNA[i - delay_tl]/(K_triggerRNA_toehold + this.triggerRNA[i - delay_tl]))) -
				(Math.log(2.0)/HL_sigmaC)*this.sigmaC[i]);
			this.antiA[i + 1] = this.antiA[i] + delta*(rTRANSLATION_RATE*this.mRNA_antiA[i - delay_tl] -
				(Math.log(2.0)/HL_antiA)*this.antiA[i]);
			this.antiB[i + 1] = this.antiB[i] + delta*(rTRANSLATION_RATE*this.mRNA_antiB[i - delay_tl] -
				(Math.log(2.0)/HL_antiB)*this.antiB[i]);
			this.antiC[i + 1] = this.antiC[i] + delta*(rTRANSLATION_RATE*this.mRNA_antiC[i - delay_tl] -
				(Math.log(2.0)/HL_antiC)*this.antiC[i]);

			if(i%(int)(STEPS*LENGTH_OF_CYCLE) < (int)((LENGTH_OF_CYCLE - LENGTH_OF_PNRD)*STEPS)) {
				this.triggerRNA[i + 1] = this.triggerRNA[i] - delta*(Math.log(2.0)/HL_RNA)*this.triggerRNA[i];
			} else {
				this.triggerRNA[i + 1] = this.triggerRNA[i] + delta*(rTRANSCRIPTION_RATE*LOW_COPY - (Math.log(2.0)/HL_RNA)*this.triggerRNA[i]);
			}
			this.arabinose[i + 1] = this.arabinose[i] - delta*(Math.log(2.0)/HL_ARABINOSE)*this.arabinose[i];

			this.mRNA_sigmaA[i + 1] = this.mRNA_sigmaA[i] + delta*(rTRANSCRIPTION_RATE*LOW_COPY*(this.sigmaA[i - delay_tc]*(1.0 -
				this.antiA[i - delay_tc]/(K_sigmaA_antiA + this.antiA[i - delay_tc]))/(K_sigmaA_pA + this.sigmaA[i - delay_tc]*(1.0 -
				this.antiA[i - delay_tc]/(K_sigmaA_antiA + this.antiA[i - delay_tc]))) + ARAC*
				this.arabinose[i - delay_tc]/(K_araC_ARABINOSE + this.arabinose[i - delay_tc])/(K_araC_pbad + ARAC*
				this.arabinose[i - delay_tc]/(K_araC_ARABINOSE + this.arabinose[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.mRNA_sigmaA[i]);
			this.mRNA_sigmaB[i + 1] = this.mRNA_sigmaB[i] + delta*(rTRANSCRIPTION_RATE*LOW_COPY*this.sigmaB[i - delay_tc]*(1.0 -
				this.antiB[i - delay_tc]/(K_sigmaB_antiB + this.antiB[i - delay_tc]))/(K_sigmaB_pB + this.sigmaB[i - delay_tc]*(1.0 -
				this.antiB[i - delay_tc]/(K_sigmaB_antiB + this.antiB[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.mRNA_sigmaB[i]);
			this.mRNA_sigmaC[i + 1] = this.mRNA_sigmaC[i] + delta*(rTRANSCRIPTION_RATE*LOW_COPY*this.sigmaC[i - delay_tc]*(1.0 -
				this.antiC[i - delay_tc]/(K_sigmaC_antiC + this.antiC[i - delay_tc]))/(K_sigmaC_pC + this.sigmaC[i - delay_tc]*(1.0 -
				this.antiC[i - delay_tc]/(K_sigmaC_antiC + this.antiC[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.mRNA_sigmaC[i]);
			this.mRNA_antiA[i + 1] = this.mRNA_antiA[i] + delta*(rTRANSCRIPTION_RATE*HIGH_COPY*this.sigmaB[i - delay_tc]*(1.0 -
				this.antiB[i - delay_tc]/(K_sigmaB_antiB + this.antiB[i - delay_tc]))/(K_sigmaB_pB + this.sigmaB[i - delay_tc]*(1.0 -
				this.antiB[i - delay_tc]/(K_sigmaB_antiB + this.antiB[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.mRNA_antiA[i]);
			this.mRNA_antiB[i + 1] = this.mRNA_antiB[i] + delta*(rTRANSCRIPTION_RATE*HIGH_COPY*this.sigmaC[i - delay_tc]*(1.0 -
				this.antiC[i - delay_tc]/(K_sigmaC_antiC + this.antiC[i - delay_tc]))/(K_sigmaC_pC + this.sigmaC[i - delay_tc]*(1.0 -
				this.antiC[i - delay_tc]/(K_sigmaC_antiC + this.antiC[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.mRNA_antiB[i]);
			this.mRNA_antiC[i + 1] = this.mRNA_antiC[i] + delta*(rTRANSCRIPTION_RATE*HIGH_COPY*this.sigmaA[i - delay_tc]*(1.0 -
				this.antiA[i - delay_tc]/(K_sigmaA_antiA + this.antiA[i - delay_tc]))/(K_sigmaA_pA + this.sigmaA[i - delay_tc]*(1.0 -
				this.antiA[i - delay_tc]/(K_sigmaA_antiA + this.antiA[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.mRNA_antiC[i]);
			this.cr_mRNA_sigmaA[i + 1] = this.cr_mRNA_sigmaA[i] + delta*(rTRANSCRIPTION_RATE*LOW_COPY*this.sigmaC[i - delay_tc]*(1.0 -
				this.antiC[i - delay_tc]/(K_sigmaC_antiC + this.antiC[i - delay_tc]))/(K_sigmaC_pC + this.sigmaC[i - delay_tc]*(1.0 -
				this.antiC[i - delay_tc]/(K_sigmaC_antiC + this.antiC[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.cr_mRNA_sigmaA[i]);
			this.cr_mRNA_sigmaB[i + 1] = this.cr_mRNA_sigmaB[i] + delta*(rTRANSCRIPTION_RATE*LOW_COPY*this.sigmaA[i - delay_tc]*(1.0 -
				this.antiA[i - delay_tc]/(K_sigmaA_antiA + this.antiA[i - delay_tc]))/(K_sigmaA_pA + this.sigmaA[i - delay_tc]*(1.0 -
				this.antiA[i - delay_tc]/(K_sigmaA_antiA + this.antiA[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.cr_mRNA_sigmaB[i]);
			this.cr_mRNA_sigmaC[i + 1] = this.cr_mRNA_sigmaC[i] + delta*(rTRANSCRIPTION_RATE*LOW_COPY*this.sigmaB[i - delay_tc]*(1.0 -
				this.antiB[i - delay_tc]/(K_sigmaB_antiB + this.antiB[i - delay_tc]))/(K_sigmaB_pB + this.sigmaB[i - delay_tc]*(1.0 -
				this.antiB[i - delay_tc]/(K_sigmaB_antiB + this.antiB[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.cr_mRNA_sigmaC[i]);

			this.matureGFP[i + 1] = this.matureGFP[i] + delta*((Math.log(2.0)/MT_GFP)*this.immatureGFP[i] -
				(Math.log(2.0)/HL_GFP)*this.matureGFP[i]);
			this.immatureGFP[i + 1] = this.immatureGFP[i] + delta*(rTRANSLATION_RATE*this.mRNA_GFP[i - delay_tl] -
				(Math.log(2.0)/MT_GFP)*this.immatureGFP[i] - (Math.log(2.0)/HL_GFP)*this.immatureGFP[i]);
			this.matureRFP[i + 1] = this.matureRFP[i] + delta*((Math.log(2.0)/MT_RFP)*this.immatureRFP[i] -
				(Math.log(2.0)/HL_RFP)*this.matureRFP[i]);
			this.immatureRFP[i + 1] = this.immatureRFP[i] + delta*(rTRANSLATION_RATE*this.mRNA_RFP[i - delay_tl] -
				(Math.log(2.0)/MT_RFP)*this.immatureRFP[i] - (Math.log(2.0)/HL_RFP)*this.immatureRFP[i]);
			this.matureCFP[i + 1] = this.matureCFP[i] + delta*((Math.log(2.0)/MT_CFP)*this.immatureCFP[i] -
				(Math.log(2.0)/HL_CFP)*this.matureCFP[i]);
			this.immatureCFP[i + 1] = this.immatureCFP[i] + delta*(rTRANSLATION_RATE*this.mRNA_CFP[i - delay_tl] -
				(Math.log(2.0)/MT_CFP)*this.immatureCFP[i] - (Math.log(2.0)/HL_CFP)*this.immatureCFP[i]);
			this.mRNA_GFP[i + 1] = this.mRNA_GFP[i] + delta*(rTRANSCRIPTION_RATE*LOW_COPY*this.sigmaA[i - delay_tc]*(1.0 -
				this.antiA[i - delay_tc]/(K_sigmaA_antiA + this.antiA[i - delay_tc]))/(K_sigmaA_pA + this.sigmaA[i - delay_tc]*(1.0 -
				this.antiA[i - delay_tc]/(K_sigmaA_antiA + this.antiA[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.mRNA_GFP[i]);
			this.mRNA_RFP[i + 1] = this.mRNA_RFP[i] + delta*(rTRANSCRIPTION_RATE*LOW_COPY*this.sigmaB[i - delay_tc]*(1.0 -
				this.antiB[i - delay_tc]/(K_sigmaB_antiB + this.antiB[i - delay_tc]))/(K_sigmaB_pB + this.sigmaB[i - delay_tc]*(1.0 -
				this.antiB[i - delay_tc]/(K_sigmaB_antiB + this.antiB[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.mRNA_RFP[i]);
			this.mRNA_CFP[i + 1] = this.mRNA_CFP[i] + delta*(rTRANSCRIPTION_RATE*LOW_COPY*this.sigmaC[i - delay_tc]*(1.0 -
				this.antiC[i - delay_tc]/(K_sigmaC_antiC + this.antiC[i - delay_tc]))/(K_sigmaC_pC + this.sigmaC[i - delay_tc]*(1.0 -
				this.antiC[i - delay_tc]/(K_sigmaC_antiC + this.antiC[i - delay_tc]))) - (Math.log(2.0)/HL_RNA)*this.mRNA_CFP[i]);
		}
		
		new Output(this);
	}
}

class Output extends Canvas {
	private static final int width = 2500;
	private static final int height = 1250;
	private static Calculation result;

	protected Output(Calculation calculation) {
		super();
		setSize(width, height);
		setBackground(Color.white);
		setForeground(Color.black);
		result = calculation;
		Frame f = new Frame("Time Series of Fluorescence");
		f.add(this);
		f.pack();
		f.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		f.setVisible(true);
	}

	public void paint(Graphics g) {
		g.drawLine(50, 0, 50, height - 51);
		g.drawLine(50, height - 51, width - 1, height- 51);
		
		int numberOfPoints = result.matureGFP.length;
		double xlim = result.GENERATION*result.LENGTH_OF_CYCLE;
		double ylim = 0.0;
		for(int i = 0; i < numberOfPoints; i++) {
			ylim = (result.matureGFP[i] > ylim) ? result.matureGFP[i] : ylim;
			ylim = (result.matureRFP[i] > ylim) ? result.matureRFP[i] : ylim;
			ylim = (result.matureCFP[i] > ylim) ? result.matureCFP[i] : ylim;
		}

		for(int i = 0; i < numberOfPoints; i++) {
			int fix_time = (int)(50 + ((i/(double)result.STEPS)/xlim)*(width - 50));
			int fix_GFP = (int)(height - 51 - (result.matureGFP[i]/ylim)*(height - 50));
			int fix_RFP = (int)(height - 51 - (result.matureRFP[i]/ylim)*(height - 50));
			int fix_CFP = (int)(height - 51 - (result.matureCFP[i]/ylim)*(height - 50));
			g.setColor(Color.green);
			g.fillRect(fix_time, fix_GFP, 1, 1);
			g.setColor(Color.red);
			g.fillRect(fix_time, fix_RFP, 1, 1);
			g.setColor(Color.blue);
			g.fillRect(fix_time, fix_CFP, 1, 1);
		}

		g.setColor(Color.black);
		for(int i = 0; i < 10; i++) {
			g.drawString(String.valueOf((float)((i/10.0)*xlim)), 50 + i*(width - 50)/10, height - 41);
			g.drawString(String.valueOf((float)((i/10.0)*ylim)), 40, height - 51 - i*(height - 51)/10);
		}
	}
}

public class Simulation {
	public static void main(String[] args) {
		new Calculation();
	}
}

